﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;

using CMS.SettingsProvider;
using CMS.GlobalHelper;
using CMS.OnlineMarketing;
using CMS.Ecommerce;
using CMS.EcommerceProvider;

/// <summary>
/// Summary description for CustomMacroMethods
/// </summary>
public class CustomMacroMethods
{
    /// <summary>
    /// Find any shopping carts for current Contact.
    /// </summary>
    public static bool OldShoppingCarts()
    {
        return OldShoppingCarts(OnlineMarketingContext.CurrentContact, DateTime.Now);
    }

    /// <summary>
    /// Find any shopping carts for specified contact.
    /// </summary>
    public static bool OldShoppingCarts(ContactInfo contact)
    {
        return OldShoppingCarts(contact, DateTime.Now);
    }

    /// <summary>
    /// Find shopping carts older than specific time for a contact.
    /// </summary>
    public static bool OldShoppingCarts(ContactInfo contact, DateTime when)
    {
        if (contact != null)
        {
            DataSet ds = null;
            string where = "";

            ds = MembershipInfoProvider.GetRelationships("MemberType = " + (int)MemberTypeEnum.EcommerceCustomer + " AND ActiveContactID = " + contact.ContactID, "MembershipCreated");
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                where = SqlHelperClass.AddWhereCondition(where, "ShoppingCartCustomerID in (" + TextHelper.Join(",", SystemDataHelper.GetStringValues(ds.Tables[0], "RelatedID")) + ")");
            }

            ds = MembershipInfoProvider.GetRelationships("MemberType = " + (int)MemberTypeEnum.CmsUser + " AND ActiveContactID = " + contact.ContactID, "MembershipCreated");
            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                where = SqlHelperClass.AddWhereCondition(where, "ShoppingCartUserID in (" + TextHelper.Join(",", SystemDataHelper.GetStringValues(ds.Tables[0], "RelatedID")) + ")", "OR");
            }

            if (where != string.Empty)
            {
                where = SqlHelperClass.AddWhereCondition(where, "ShoppingCartLastUpdate < \'" + when + "\'");

                InfoDataSet<ShoppingCartInfo> shoppingCarts = ShoppingCartInfoProvider.GetShoppingCarts(where, "ShoppingCartLastUpdate DESC");
                if (!DataHelper.DataSourceIsEmpty(shoppingCarts))
                {
                    return true;
                }
            }
        }
        return false;
    }

    /// <summary>
    /// Wrapper for OldShoppingCarts methods, suitable for registration as a macro method.
    /// </summary>
    /// <param name="parameters">Parameters of the method</param>
    public static object OldShoppingCarts(params object[] parameters)
    {
        switch (parameters.Length)
        {
            case 0:
                return OldShoppingCarts();
            case 1:
                return OldShoppingCarts(parameters[0] as ContactInfo);
            case 2:
                return OldShoppingCarts(parameters[0] as ContactInfo, ValidationHelper.GetDateTime(parameters[1], DateTime.Now));
            default :
                throw new NotSupportedException();
        }
    }

    public static void RegisterMethods()
    {
        // Registers OldShoppingCarts with two parameter overloads.
        MacroMethod oldShoppingCarts = new MacroMethod("OldShoppingCarts", OldShoppingCarts)
        {
            Comment = "Returns Shopping Cart Objects for the Contact older than specified DateTime.",
            Type = typeof(string),
            AllowedTypes = new List<Type>() { typeof(DateTime) },
            MinimumParameters = 0
        };
        oldShoppingCarts.AddParameter("contact", typeof(ContactInfo), "ContactInfo object.");
        oldShoppingCarts.AddParameter("when", typeof(DateTime), "Specific DateTime.");

        MacroMethods.RegisterMethod(oldShoppingCarts);

        // Registers a code snippet for OldShoppingCarts.
        MacroMethod oldShoppingCartsSnippet = new MacroMethod("OldShoppingCartsSnippet", OldShoppingCarts)
        {
            Comment = "Calls OldShoppingCarts to retrieve all shopping carts older than 30 minutes.",
            Type = typeof(string),
            AllowedTypes = new List<Type>() { typeof(string) },
            Snippet = "OldShoppingCarts(ECommerceContext.Contact, ToLower(DateTime.Now.AddMinutes(-30)));"
        };
        MacroMethods.RegisterMethod(oldShoppingCartsSnippet);
    }

}